//Acknowlwledgement:
// H-bridge D11C hello-world
// Neil Gershenfeld 11/8/22

// Modified by Maxime Richard

#define output(pin) (PORT->Group[0].DIRSET.reg = pin)
#define set(pin) (PORT_IOBUS->Group[0].OUTSET.reg = pin)
#define clear(pin) (PORT_IOBUS->Group[0].OUTCLR.reg = pin)
#define PWM_count 20 // number of PWM cycles
#define on_delay_slow() delayMicroseconds(20) // PWM on time
#define off_delay() delayMicroseconds(10) // PWM off time

#define IN1 PORT_PA15 // IN1
#define IN2 PORT_PA14 // IN2

//target frame rate
#define numberMagnets 10
#define numberFrames 20
#define targetFrameRate = 12;
uint16_t targetInterval = 1000 / 12;
uint16_t targetSignalInterval = numberFrames / numberMagnets * targetInterval;
uint32_t averageInterval = 1000;
uint32_t currentFrameInterval = 1000;


//Hall effect input device
#define input(pin) (PORT->Group[0].DIRCLR.reg = pin)
#define inputPin 4
#define numberSamples 2
#define thresholdSignal (520 * numberSamples)
#define deactivationThreshold (510 * numberSamples)

//frequency calculation
uint16_t beforePreviousInterval = averageInterval;
uint16_t previousInterval = averageInterval;
uint16_t currentInterval = averageInterval;
uint32_t lastThreshold = 0;
bool aboveThreshold= false;

uint16_t currentPulseWidth = 5;

//LED flash
uint32_t lastFlash = 0;
#define LEDPORT PORT_PA02
#define flashLength 10

//DEBUG
uint32_t stopWatch = 0;

void setup() {
  output(IN1);
  output(IN2);
  output(LEDPORT);
  pinMode(inputPin, INPUT);
  clear(IN2);

  Serial.begin(9600);
}

void loop() {
  stopWatch = micros();
  //turn on LEDs
  uint32_t currentTime = millis();
  bool ledOn = false;
  if ((currentTime - lastFlash) > currentFrameInterval) {
     lastFlash = currentTime;
     set(LEDPORT);
     ledOn = true;
  }

  //drive motor
  for (int i = 0; i < PWM_count; i++ )
  {
    set(IN1);
    delayMicroseconds(30);
    clear(IN1);
    off_delay();
  }



  //process input
  uint32_t inputSignal = 0;

  for (int i = 0; i < numberSamples; i++) {
     inputSignal += analogRead(inputPin);
  }
  //Serial.println(inputSignal);

  if (inputSignal > thresholdSignal) {
    if (!aboveThreshold) {
      Serial.println("threshold!");
      aboveThreshold = true;
      uint32_t current = millis();
      beforePreviousInterval = previousInterval;
      previousInterval = currentInterval;
      currentInterval = current - lastThreshold;
      lastThreshold = current;

      averageInterval = (beforePreviousInterval + previousInterval + currentInterval) / 3;
      currentFrameInterval = averageInterval / 2;
    }
  } else if (inputSignal < deactivationThreshold) {
    aboveThreshold = false;
  }

  if (ledOn) {
    clear(LEDPORT);
  }


  Serial.print("cycle duration: ");
  Serial.println((micros() - stopWatch));
}
